/// Program ERCK_2WD_PratiLiniju
// Eduardo robotic wheelchair ERCK_2WD follow a black line drawn on the floor
// The black line should be around 3.5cm thick, a radius of curvature greater than 30cm
// Required components Eduardo kit: EMB, ERS, ERCK_2WD, EOSK-3 

// Note: Black gives a low analogue reading (low reflectance), white gives a high analogue reading (high reflectance)

// Pins optical sensor
#define LEVI_SENZOR        0
#define SREDNJI_SENZOR     1  
#define DESNI_SENZOR       2
// Pins for speed and direction of the motor
#define LEVI_MOTOR_SMER    7
#define DESNI_MOTOR_SMER   8
#define LEVI_MOTOR_BRZINA  9
#define DESNI_MOTOR_BRZINA 10
// Pin for LED indicator
#define LED_INDIKATOR      13

// Start the speed and rotation speed trolleys
// EDIT initial speed IF TROLLEY GO SLOW
// EDIT SPEED ROTATION IF TROLLEY PREBRUZO or too slow REACT
#define BRZINA_POCETNA     75 //150 // Forwards Speed      
#define BRZINA_ROTACIJE    50 // Rotation Speed  

#define THRESHOLD_CALIB 3 // Smaller to make the unit more sensitive, larger to make it less sensitive ("original" value of 10)

int PragLevogSenzora = 0;  // Thresholds decision          
int PragSrednjegSenzora = 0;
int PragDesnogSenzora = 0;   

int BrzinaLevo;
int BrzinaDesno;
int stanjeSenzora;

// Calibration of the sensor signal measured with all three optical sensors 
// When it is placed in front of them bright area. 
// Half of this amount (50%) is set as the threshold decision, 
// Because the sensor signal with the default dark area more than 50% lower 
// Signals from sensors mounted light povesinom
void kalibracija_senzora() 
{
  for (int x=0; x<10; x++) // Meriti deset puta i akumulirati rezultate
  { 
    digitalWrite(LED_INDIKATOR, HIGH); // Upaliti LED indikator
    PragLevogSenzora = PragLevogSenzora + analogRead(LEVI_SENZOR);           // Akumulirati vrednosti za levi senzor
    PragSrednjegSenzora = PragSrednjegSenzora + analogRead(SREDNJI_SENZOR);  // Akumulirati vrednosti za srednji senzor
    PragDesnogSenzora = PragDesnogSenzora + analogRead(DESNI_SENZOR);        // Akumulirati vrednosti za desni senzor
    delay(100);
    digitalWrite(LED_INDIKATOR, LOW); // Ugasiti LED indikator
    delay(100);
  }
  // Accumulated results divided by 10 to obtain the mean value
  // By giving 2 to give a decision threshold for sensors
  PragLevogSenzora = PragLevogSenzora / (10 + THRESHOLD_CALIB);
  PragSrednjegSenzora = PragSrednjegSenzora / (10 + THRESHOLD_CALIB);
  PragDesnogSenzora = PragDesnogSenzora / (10 + THRESHOLD_CALIB);
}

void setup()
{
  // Podesiti signale za upravljanje motorima kao izlaze
  pinMode(LED_INDIKATOR, OUTPUT);             
  pinMode(LEVI_MOTOR_BRZINA, OUTPUT);
  pinMode(DESNI_MOTOR_BRZINA, OUTPUT);
  pinMode(LEVI_MOTOR_SMER, OUTPUT);
  pinMode(DESNI_MOTOR_SMER, OUTPUT);
    
  kalibracija_senzora();              // Calibrate optical sensors 
  delay(3000);                        // sacekati 3 sekunde
  digitalWrite(LED_INDIKATOR, HIGH);  // Upaliti LED indikator
  delay(100);                         // sacekati 0.1 sekundu
  
  // Set-directional motor for driving forward
  digitalWrite(LEVI_MOTOR_SMER, HIGH);
  digitalWrite(DESNI_MOTOR_SMER, HIGH);
}

void loop() 
{
  stanjeSenzora = 111;
  if(analogRead(LEVI_SENZOR) > PragLevogSenzora) stanjeSenzora = stanjeSenzora - 100;
  if(analogRead(SREDNJI_SENZOR) > PragSrednjegSenzora) stanjeSenzora = stanjeSenzora - 10;
  if(analogRead(DESNI_SENZOR) > PragDesnogSenzora) stanjeSenzora = stanjeSenzora - 1;

  if(stanjeSenzora == 100)                                  // If only the Left sensor above the black line to turn left!
  {
    analogWrite(LEVI_MOTOR_BRZINA, BRZINA_POCETNA - BRZINA_ROTACIJE);
    analogWrite(DESNI_MOTOR_BRZINA, BRZINA_POCETNA + BRZINA_ROTACIJE);
  }

  else if(stanjeSenzora == 001)        // If only Sensor Right above the black line to turn right!
  {
    analogWrite(LEVI_MOTOR_BRZINA, BRZINA_POCETNA + BRZINA_ROTACIJE);
    analogWrite(DESNI_MOTOR_BRZINA, BRZINA_POCETNA - BRZINA_ROTACIJE);
  } 
  else if((stanjeSenzora == 111) || (stanjeSenzora == 000)) // If all the sensors, it means that there is no line. Stop!             
  {                                                         
    analogWrite(LEVI_MOTOR_BRZINA, 0);                      
    analogWrite(DESNI_MOTOR_BRZINA, 0);  
  } 
  else                                                      // In other cases, continue straight                                                                                      
  {                                                    
    analogWrite(LEVI_MOTOR_BRZINA, BRZINA_POCETNA);
    analogWrite(DESNI_MOTOR_BRZINA, BRZINA_POCETNA);
  }
}

