#include "Arduino.h"
#include "TurtleLineHunter.h"

TurtleLineHunter::TurtleLineHunter(int _leftPin, int _centrePin, int _rightPin)
    : leftPin(_leftPin)
    , centrePin(_centrePin)
    , rightPin(_rightPin)
 {
 }

void TurtleLineHunter::setup() const
 { pinMode(leftPin, INPUT);
   pinMode(centrePin, INPUT);
   pinMode(rightPin, INPUT);
 }

// There are three line hunting sensors. Each provides a
// single bit of HIGH if it is over a dark line. If we
// arrange the bits such that the left bit is the LSB and
// the right is the MSB then we have a 3-bit number.
// 0b000 - no line found
// 0b001 - line on edge of right side
// 0b010 - line dead centre
// 0b011 - line on right and centre
// 0b100 - line on edge of left side
// 0b101 - two lines found on both side (unusual case)
// 0b110 - line on left and centre
// 0b111 - most likely are perpendicular to line
int TurtleLineHunter::readBits() const
 { int output = 0b000;
   output |= (digitalRead(rightPin)  << 0);
   output |= (digitalRead(centrePin) << 1);
   output |= (digitalRead(leftPin)   << 2);
   return output;
 }

// Reads all three sensors and returns a number based
// upon the position of the line relative to the turtle
int TurtleLineHunter::readPos(bool &foundLine) const
 { int bits = readBits();
   foundLine = bits;

   switch (bits)
    { case 0b100:
         return -2;
      case 0b110:
         return -1;
      case 0b010:
      case 0b111:
         return 0;
      case 0b011:
         return +1;
      case 0b001:
         return +2;

      default:
         return 0;
    }
 }
