#include <math.h>

#include "Arduino.h"
#include "TurtleDriver.h"

#define limit(value, min, max) \
   value < min ? min : (value > max ? max : value)


TurtleDriver::TurtleDriver(int IN1, int IN2, int IN3, int IN4, int ENA, int ENB)
    : rightWheelPin1(IN1)
    , rightWheelPin2(IN2)
    , leftWheelPin1(IN3)
    , leftWheelPin2(IN4)
    , rightWheelSpeedPin(ENA)
    , leftWheelSpeedPin(ENB)
    , leftWheelSpeed(0.0)
    , rightWheelSpeed(0.0)
 {
 }

void TurtleDriver::setup() const
 { pinMode(leftWheelPin1, OUTPUT);
   pinMode(leftWheelPin2, OUTPUT);
   pinMode(rightWheelPin1, OUTPUT);
   pinMode(rightWheelPin2, OUTPUT);

   pinMode(leftWheelSpeedPin, OUTPUT);
   pinMode(rightWheelSpeedPin, OUTPUT);
 }

void TurtleDriver::updateWheels() const
 { int leftSpeed = leftWheelSpeed * 255.0;
   analogWrite(leftWheelSpeedPin, abs(leftSpeed));
   digitalWrite(leftWheelPin1, leftSpeed >= 0 ? HIGH : LOW);
   digitalWrite(leftWheelPin2, leftSpeed <= 0 ? HIGH : LOW);      
   
   int rightSpeed = rightWheelSpeed * 255.0;
   analogWrite(rightWheelSpeedPin, abs(rightSpeed));
   digitalWrite(rightWheelPin1, rightSpeed >= 0 ? LOW : HIGH);
   digitalWrite(rightWheelPin2, rightSpeed <= 0 ? LOW : HIGH);
 }

void TurtleDriver::testDrive()
 { //straight
   setWheelSpeed(0.5, 0.5);
   delay(2000); 

   // Back
   setWheelSpeed(-0.5, -0.5);
   delay(2000); // left

   // spin clockwise
   setWheelSpeed(0.3, -0.3);
   delay(2000); // turn right

   // spin clockwise
   setWheelSpeed(-0.3, +0.3);
   delay(2000); // turn right

   // brake
   setWheelSpeed(0.0, 0.0);
   delay(5000);
 }

int TurtleDriver::setWheelSpeed(float leftSpeed, float rightSpeed)
 { leftWheelSpeed = leftSpeed;//limit(leftSpeed, -1.0, +1.0);
   rightWheelSpeed = rightSpeed; //limit(rightSpeed, -1.0, +1.0);
   updateWheels();
 }